//////////////////////////////////////////////////////////////////////////
//
// BaseAutomaton.h 
// Provides BaseAutomaton class implementation 
//
//////////////////////////////////////////////////////////////////////////

#pragma once 
#include <boost/optional.hpp>
#include "trace.h"

// provides asynchronous events mechanism 
#define WM_EVENT        WM_USER + 1 
#define WM_CALCSTATE    WM_USER + 2

//////////////////////////////////////////////////////////////////////////
//
// BaseAutomaton class, provides asynchronous events mechanism and
// callbacks on state changes 
//

template< class State, class Event >
   struct BaseAutomaton 
      : public    CWindowImpl< BaseAutomaton >  
      , protected TracerImpl
{
public: 
   void Init( ITracer* tracer ) 
   {
      TracerImpl::Init( tracer ) ;

      // window creation 
      if ( IsWindow())
         DestroyWindow() ;

      Create( NULL, rcDefault, NULL, WS_POPUP ) ;
      ShowWindow( FALSE ) ;
      
      // automaton initialization 
      state_.reset( initialState_ ) ;
      Trace( L"Automaton started at state: %d", initialState_ ) ;
      OnStateChanged() ;
   }

   void DumpState() 
   {
      Trace( L"Current state: %d", initialState_ ) ;
   }

   BOOL ProcessEvent( Event event )
   {
      if ( IsInputEvent( event ))
      {
         PostMessage( WM_EVENT, event, 0 ) ;
         return TRUE ;
      }

      return FALSE ;
   }

protected:
   State GetState() const { return *state_  ; }

// window routine 
private:
   BEGIN_MSG_MAP( BaseAutomaton )
      MESSAGE_HANDLER( WM_EVENT     , OnEventReceived )  
      MESSAGE_HANDLER( WM_CALCSTATE , OnCalcState     )
   END_MSG_MAP  ()

   LRESULT OnEventReceived( UINT, WPARAM wParam, LPARAM, BOOL& )
   {
      OnEvent( static_cast< Event > ( wParam )) ;
      return 0 ;
   }
   
   LRESULT OnCalcState( UINT, WPARAM, LPARAM, BOOL& )
   {
      CalcState() ;
      return 0 ;
   }



// automaton routine 
protected:

   typedef boost::optional< State > OptState ;
   typedef boost::optional< Event > OptEvent ;

   // defined by derived class 
   virtual void CalcState     ( OptState& state ) = 0 ;
   virtual void OnStateChanged()                  = 0 ;
   virtual BOOL IsInputEvent  ( Event event ) { return TRUE ; }

protected:
   BaseAutomaton( State initialState )
      : initialState_( initialState )
   {
   }

   ~BaseAutomaton()
   {
      if( IsWindow())
         DestroyWindow() ;
   }

   // helps to work with events 
   BOOL e( long eventNumber ) const { return event_ && ( *event_ == eventNumber ) ; }

   void CalcState()
   {
      OptState state ;

      CalcState( state ) ;
      event_.reset() ;

      if ( state )
      {
         state_.reset( *state ) ;
         Trace( L"Entered state: %d", *state_ ) ;
         OnStateChanged() ;

         PostMessage( WM_CALCSTATE, 0, 0 ) ; // to avoid recursion 
      }
   }


protected:
   // notification on new event 
   virtual BOOL OnEvent( Event event ) 
   {
      event_.reset( event ) ;
      CalcState() ;

      return TRUE ;
   }

protected:
   OptEvent event_        ;

private:
   State    initialState_ ;
   OptState state_        ;
};