#pragma once 
#include "Interfaces.h"

#include "FileList.h"
#include "ServerList.h"
#include "UserList.h"

#include <algorithm>

class LinkHolder 
   : public CComObjectRootEx< CComMultiThreadModel >
   , public ILink 
{
public:
   // COM routine 
   BEGIN_COM_MAP( LinkHolder )
      COM_INTERFACE_ENTRY( ILink )
   END_COM_MAP  ()
   
   void Init( FilesParser::ItemEntry const* pItem )
   {
      pItem_ = pItem ;
   }

   // ILink 
   string const& __stdcall GetImageName() 
   {
      return pItem_->name ;
   }
   string const& __stdcall GetFtpServer()
   {
      return pItem_->server ;
   }
   string const& __stdcall GetFtpLink() 
   {
      return pItem_->link ;
   }
   BOOL __stdcall IsFolder() 
   {
      return pItem_->folder ;
   }
   long __stdcall GetSize()
   {
      return pItem_->size ;
   }
   void __stdcall GetTime( short& year, short& month, short& date, 
                           short& hour, short& minute ) 
   {
      year   = pItem_->year   ;
      month  = pItem_->month  ;
      date   = pItem_->date   ;
      hour   = pItem_->hour   ;
      minute = pItem_->minute ;
   }

private:
   FilesParser::ItemEntry const* pItem_ ;
};

class LinksIterator
   : public CComObjectRootEx< CComMultiThreadModel > 
   , public ILinksIterator
{
public:
   // COM routine 
   BEGIN_COM_MAP( LinksIterator )
      COM_INTERFACE_ENTRY( ILinksIterator  )
   END_COM_MAP  ()

   void Init( FilesParser::Items const* pItems )
   {
      pItems_ = pItems ;
      Reset() ;
   }

   // ILinksIterator
   void __stdcall Reset() 
   {
      it_ = pItems_->begin() ;
   }
   ILinkPtr  __stdcall Next () 
   {
      if( it_ != pItems_->end())
      {
         CComObject< LinkHolder >* p ;
         CComObject< LinkHolder >::CreateInstance( &p ) ;
         if( p )
         {
            p->Init( &(*it_) ) ;
            ++ it_ ;
            return p ;
         }
      }

      return NULL ;
   }

private:
   FilesParser::Items::const_iterator it_ ;
   FilesParser::Items const*          pItems_ ;
};

class LinksProvider 
   : public CComObjectRootEx< CComMultiThreadModel >
   , public ILinksProvider
   , public IAuthentication
{
   // COM routine 
   BEGIN_COM_MAP( LinksProvider )
     COM_INTERFACE_ENTRY( ILinksProvider  )
     COM_INTERFACE_ENTRY( IAuthentication )
   END_COM_MAP  ()

   LinksProvider()
      : fp_ ( "Config\\files.cfg"  )
      , sp_ ( "Config\\servers.cfg")
      , up_ ( "Config\\users.cfg"  )
   {
   }

//ILinksProvider
   BOOL __stdcall GetFtpAccount( string const& ftpServer, LPWSTR user, LPWSTR pass, size_t& size ) 
   {
      ServersParser::ServerList const& list = sp_.GetServerList() ;
      ServersParser::ServerList::const_iterator it = list.find( ftpServer ) ;

      if( it != list.end())
      {
         size_t maxSize = max( it->second.user.size(), it->second.pass.size()) ; 

         if ( maxSize > size )
         {
            size = maxSize ;
            return FALSE ;
         }
          
         swprintf( user, L"%s", it->second.user.c_str()) ;
         swprintf( pass, L"%s", it->second.pass.c_str()) ;

         return TRUE ;
      }

      return FALSE ;
   }

   ILinksIteratorPtr __stdcall CreateItemIterator( string const& folderName ) 
   {
      FilesParser::FolderList const& list = fp_.GetFolderList() ;
      FilesParser::FolderList::const_iterator it = list.find( folderName ) ;

      if( it != list.end())
      {
         CComObject< LinksIterator >* p ;
         CComObject< LinksIterator >::CreateInstance( &p ) ;
         if( p )
         {
            p->Init( &( it->second )) ;
            return p ;
         }
      }

      return NULL ;
   }

   ILinkPtr __stdcall Search( string const& fileName )
   {
      size_t pos = fileName.find_last_of( L"/" ) ;
      
      if ( pos == std::wstring::npos )
         return NULL ;

      string folder ;

      if ( pos == 0 )
         folder = L"/" ;
      else 
      {
         folder = fileName  ;
         folder.erase( pos ) ;
      }

      string file   = string( fileName, pos + 1 ) ;

      FilesParser::FolderList const& list = fp_.GetFolderList() ;
      FilesParser::FolderList::const_iterator it = list.find( folder ) ;

      FilesParser::Items::const_iterator item = std::find( it->second.begin(), it->second.end(), file ) ;
      if( item != it->second.end())
      {
         CComObject< LinkHolder >* p ;
         CComObject< LinkHolder >::CreateInstance( &p ) ;
         if( p )
         {
            p->Init( &( *item )) ;
            return p ;
         }
      }

      return NULL ;
   }

// IAuthentication
   USER_PRIVILEGE __stdcall CheckUser   ( string const& user ) 
   {
      UsersParser::UserList const& list = up_.GetUserList() ;
      UsersParser::UserList::const_iterator it = list.find( user ) ;

      if( it != list.end())
      {
         if( it->second == L"" )
            return UP_NO_PASW ;
         else 
            return UP_NEED_PASW ;
      }

      return UP_NO_USER ;
   }   

   BOOL __stdcall CorrectPassw( string const& user, string const& passw ) 
   {
      UsersParser::UserList const& list = up_.GetUserList() ;
      UsersParser::UserList::const_iterator it = list.find( user ) ;

      if( it != list.end())
         return it->second == passw ;

      return FALSE ;
   }

private:
   FilesParser    fp_ ;
   ServersParser  sp_ ;
   UsersParser    up_ ;
};